@@ -509,7 +509,12 @@ sub parse_dep5 {
                             }
 
                         } else {
-                            my $dir = $processable->orig->lookup($wc_value);
+                            my $dir;
+                            if ($processable->native) {
+                                $dir= $processable->patched->lookup($wc_value);
+                            } else {
+                                $dir = $processable->orig->lookup($wc_value);
+                            }
                             if ($dir) {
                                 my @files = grep { $_->is_file }
                                   $dir->children('breadth-first');
