@@ -41,7 +41,10 @@ sub source {
     my $processable = $self->processable;
 
     # Don't check package if it doesn't contain a .php file
-    if (none { $_->basename =~ m/\.php$/i } $processable->sorted_index) {
+    if (
+        none { $_->basename =~ m/\.php$/i && !$_->is_dir }
+        $processable->patched->sorted_list
+    ){
         return;
     }
 
@@ -49,8 +52,8 @@ sub source {
     my $package_type = 'unknown';
 
     # PEAR or PECL package
-    my $package_xml = $processable->index('package.xml');
-    my $package2_xml = $processable->index('package2.xml');
+    my $package_xml = $processable->patched->lookup('package.xml');
+    my $package2_xml = $processable->patched->lookup('package2.xml');
     if (defined($package_xml) || defined($package2_xml)) {
         # Checking source builddep
         if (!$bdepends->implies('pkg-php-tools')) {
@@ -116,14 +119,14 @@ sub source {
         }
     }
     # PEAR channel
-    my $channel_xml = $processable->index('channel.xml');
+    my $channel_xml = $processable->patched->lookup('channel.xml');
     if (defined($channel_xml)) {
         if (!$bdepends->implies('pkg-php-tools')) {
             $self->tag('pear-channel-without-pkg-php-tools-builddep');
         }
     }
     # Composer package
-    my $composer_json = $processable->index('composer.json');
+    my $composer_json = $processable->patched->lookup('composer.json');
     if (   !defined($package_xml)
         && !defined($package2_xml)
         && defined($composer_json)) {
@@ -139,7 +142,7 @@ sub source {
             || defined($channel_xml)
             || defined($composer_json))
     ) {
-        my $rules = $processable->index_resolved_path('debian/rules');
+        my $rules = $processable->patched->resolve_path('debian/rules');
         if ($rules and $rules->is_open_ok) {
             my $has_buildsystem_phppear = 0;
             my $has_addon_phppear = 0;
