@@ -40,11 +40,11 @@ use POSIX qw(:sys_wait_h);
 my $INIT_ROOT = $ENV{'LINTIAN_ROOT'};
 
 use Lintian::Data;
-use Lintian::Output qw(:messages);
 use Lintian::Inspect::Changelog;
 use Lintian::Internal::FrontendUtil
   qw(default_parallel sanitize_environment open_file_or_fd);
-use Lintian::Processable::Pool;
+use Lintian::Output::Standard;
+use Lintian::Pool;
 use Lintian::Profile;
 use Lintian::Tags;
 use Lintian::Util qw(internal_error parse_boolean strip safe_qx);
@@ -87,9 +87,10 @@ my %opt = (                     #hash of some flags from cmd or cfg
     'jobs'              => default_parallel(),
 );
 
-my ($experimental_output_opts, %override_count);
+my $experimental_output_opts;
 
 my (@CLOSE_AT_END, $TAGS);
+my $OUTPUT = Lintian::Output::Standard->new;
 my @certainties = qw(wild-guess possible certain);
 my (@display_level, %display_source, %suppress_tags);
 my ($action, $checks, $check_tags, $dont_check, $received_signal);
@@ -98,8 +99,6 @@ my $user_dirs = $ENV{'LINTIAN_ENABLE_USER_DIRS'} // 1;
 my $exit_code = 0;
 my $STATUS_FD;
 
-my $memory_usage = sub { 'N/A'; };
-
 # }}}
 
 # {{{ Setup Code
@@ -609,21 +608,19 @@ sub main {
             if ($_ eq 'format') {
                 if ($opts{$_} eq 'colons') {
                     require Lintian::Output::ColonSeparated;
-                    $Lintian::Output::GLOBAL
-                      = Lintian::Output::ColonSeparated->new;
+                    $OUTPUT= Lintian::Output::ColonSeparated->new;
                 } elsif ($opts{$_} eq 'letterqualifier') {
                     require Lintian::Output::LetterQualifier;
-                    $Lintian::Output::GLOBAL
-                      = Lintian::Output::LetterQualifier->new;
+                    $OUTPUT= Lintian::Output::LetterQualifier->new;
                 } elsif ($opts{$_} eq 'xml') {
                     require Lintian::Output::XML;
-                    $Lintian::Output::GLOBAL = Lintian::Output::XML->new;
+                    $OUTPUT = Lintian::Output::XML->new;
                 } elsif ($opts{$_} eq 'fullewi') {
                     require Lintian::Output::FullEWI;
-                    $Lintian::Output::GLOBAL = Lintian::Output::FullEWI->new;
+                    $OUTPUT = Lintian::Output::FullEWI->new;
                 } elsif ($opts{$_} eq 'universal') {
                     require Lintian::Output::Universal;
-                    $Lintian::Output::GLOBAL = Lintian::Output::Universal->new;
+                    $OUTPUT = Lintian::Output::Universal->new;
                 }
             }
         }
@@ -648,69 +645,33 @@ sub main {
     fatal_error('The hyperlink value must be one of "on" or "off"')
       unless $opt{'hyperlinks'} =~ /^(?:on|off)$/;
 
-    if (not defined $opt{'tag-display-limit'}) {
-        if (-t STDOUT and not $opt{'verbose'}) {
-            $opt{'tag-display-limit'}
-              = Lintian::Output::DEFAULT_INTERACTIVE_TAG_LIMIT();
-        } else {
-            $opt{'tag-display-limit'} = 0;
-        }
+    if ($opt{'verbose'} || !-t STDOUT) {
+        $opt{'tag-display-limit'} //= 0;
+    } else {
+        $opt{'tag-display-limit'} //= 4;
     }
 
     if ($opt{'debug'}) {
         $opt{'verbose'} = 1;
         $ENV{'LINTIAN_DEBUG'} = $opt{'debug'};
         $SIG{__DIE__} = sub { Carp::confess(@_) };
-        if ($opt{'debug'} > 2) {
-            eval {
-                require Devel::Size;
-                Devel::Size->import(qw(total_size));
-                {
-                    no warnings qw(once);
-                    # Disable warnings about stuff Devel::Size cannot
-                    # give reliable sizes for.
-                    $Devel::Size::warn = 0;
-                }
-
-                $memory_usage = sub {
-                    my ($obj) = @_;
-                    my $size = total_size($obj);
-                    my $unit = 'B';
-                    if ($size > 1536) {
-                        $size /= 1024;
-                        $unit = 'kB';
-                        if ($size > 1536) {
-                            $size /= 1024;
-                            $unit = 'MB';
-                        }
-                    }
-                    return sprintf('%.2f %s', $size, $unit);
-                };
-                print "N: Using Devel::Size to debug memory usage\n";
-            };
-            if ($@) {
-                print "N: Cannot load Devel::Size ($@)\n";
-                print "N: Running memory usage will not be checked.\n";
-            }
-        }
     } else {
         # Ensure verbose has a defined value
-        $opt{'verbose'} = 0 unless defined($opt{'verbose'});
+        $opt{'verbose'} //= 0;
     }
 
-    $Lintian::Output::GLOBAL->verbosity_level($opt{'verbose'});
-    $Lintian::Output::GLOBAL->debug($opt{'debug'});
+    $OUTPUT->verbosity_level($opt{'verbose'});
+    $OUTPUT->debug($opt{'debug'});
 
-    $Lintian::Output::GLOBAL->color($opt{'color'});
-    $Lintian::Output::GLOBAL->tty_hyperlinks($hyperlinks_capable
-          && $opt{hyperlinks} eq 'on');
-    $Lintian::Output::GLOBAL->tag_display_limit($opt{'tag-display-limit'});
-    $Lintian::Output::GLOBAL->showdescription($opt{'info'});
+    $OUTPUT->color($opt{'color'});
+    $OUTPUT->tty_hyperlinks($hyperlinks_capable&& $opt{hyperlinks} eq 'on');
+    $OUTPUT->tag_display_limit($opt{'tag-display-limit'});
+    $OUTPUT->showdescription($opt{'info'});
 
-    $Lintian::Output::GLOBAL->perf_debug($opt{'perf-debug'});
+    $OUTPUT->perf_debug($opt{'perf-debug'});
     if (defined(my $perf_log = $opt{'perf-output'})) {
         my $fd = open_file_or_fd($perf_log, '>');
-        $Lintian::Output::GLOBAL->perf_log_fd($fd);
+        $OUTPUT->perf_log_fd($fd);
 
         push(@CLOSE_AT_END, [$fd, $perf_log]);
     }
@@ -724,8 +685,6 @@ sub main {
         open($STATUS_FD, '>', '/dev/null');
     }
 
-    $Lintian::Output::GLOBAL->print_first();
-
     # check for arguments
     if (    $action =~ /^(?:check|unpack)$/
         and $#ARGV == -1
@@ -746,12 +705,12 @@ sub main {
         my $banner = lintian_banner();
         # Print Debug banner, now that we're finished determining
         # the values and have Lintian::Output available
-        debug_msg(
+        $OUTPUT->debug_msg(
             1,$banner,
             "Lintian root directory: $INIT_ROOT",
             "Configuration file: $opt{'LINTIAN_CFG'}",
             'UTF-8: ✓ (☃)',
-            delimiter(),
+            $OUTPUT->delimiter,
         );
     }
 
@@ -761,7 +720,7 @@ sub main {
     # Ensure $opt{'LINTIAN_PROFILE'} is defined
     $opt{'LINTIAN_PROFILE'} = $PROFILE->name
       unless defined($opt{'LINTIAN_PROFILE'});
-    v_msg('Using profile ' . $PROFILE->name . '.');
+    $OUTPUT->v_msg('Using profile ' . $PROFILE->name . '.');
     Lintian::Data->set_vendor($PROFILE);
 
     $TAGS = Lintian::Tags->new;
@@ -803,7 +762,7 @@ sub main {
             chomp $line;
 
             next
-              if $line =~ /^\s*/;
+              if $line =~ /^\s*$/;
 
             push(@subjects, $line);
         }
@@ -814,7 +773,7 @@ sub main {
           unless fileno($fd) == fileno(STDIN);
     }
 
-    my $pool = Lintian::Processable::Pool->new;
+    my $pool = Lintian::Pool->new;
 
     for my $path (@subjects) {
 
@@ -829,7 +788,7 @@ sub main {
 
         eval {
             # create a new group
-            my $group = Lintian::Processable::Group->new;
+            my $group = Lintian::Group->new;
             $group->pooldir($pool->basedir);
             $group->init_from_file($absolute);
 
@@ -842,82 +801,18 @@ sub main {
     }
 
     if ($pool->empty) {
-        v_msg('No packages selected.');
+        $OUTPUT->v_msg('No packages selected.');
         exit $exit_code;
     }
 
     $ENV{INIT_ROOT} = $INIT_ROOT;
 
-    $pool->process(
-        $action, $PROFILE,$TAGS,
-        \$exit_code,\%override_count, \%opt,
-        $memory_usage,$STATUS_FD, \@unpack_info
-    );
+    $pool->process($action, $PROFILE,$TAGS,\$exit_code, \%opt,
+        $STATUS_FD, \@unpack_info, $OUTPUT);
 
     retrigger_signal()
       if $received_signal;
 
-    if (    $action eq 'check'
-        and not $opt{'no-override'}
-        and not $opt{'show-overrides'}) {
-        my $errors = $override_count{errors} || 0;
-        my $warnings = $override_count{warnings} || 0;
-        my $info = $override_count{info} || 0;
-        my $total = $errors + $warnings + $info;
-        my $unused = $TAGS->{unused_overrides};
-        if ($total > 0 or $unused > 0) {
-            my $text
-              = ($total == 1)
-              ? "$total tag overridden"
-              : "$total tags overridden";
-            my @output;
-            if ($errors) {
-                push(@output,
-                    ($errors == 1) ? "$errors error" : "$errors errors");
-            }
-            if ($warnings) {
-                push(@output,
-                    ($warnings == 1)
-                    ? "$warnings warning"
-                    : "$warnings warnings");
-            }
-            if ($info) {
-                push(@output, "$info info");
-            }
-            if (@output) {
-                $text .= ' (' . join(', ', @output). ')';
-            }
-            if ($unused == 1) {
-                $text .= "; $unused unused override";
-            } elsif ($unused > 1) {
-                $text .= "; $unused unused overrides";
-            }
-            msg($text);
-        }
-    }
-
-    my $ign_over = $TAGS->ignored_overrides;
-    if (keys %$ign_over) {
-        msg(
-            join(q{ },
-                'Some overrides were ignored,',
-                'since the tags were marked "non-overridable".'));
-        if ($opt{'verbose'}) {
-            v_msg(
-                join(q{ },
-                    'The following tags were "non-overridable"',
-                    'and had at least one override'));
-            foreach my $tag (sort keys %$ign_over) {
-                v_msg("  - $tag");
-            }
-        } else {
-            msg('Use --verbose for more information.');
-        }
-    }
-
-    # universal format sorts output from all packages before printing here
-    $Lintian::Output::GLOBAL->print_last();
-
     # }}}
 
     exit $exit_code;
@@ -1271,7 +1166,7 @@ sub retrigger_signal {
     # Re-kill ourselves with the same signal to ensure that the exit
     # code reflects that we died by a signal.
     local $SIG{$received_signal} = \&_die_in_signal_handler;
-    debug_msg(2, "Retriggering signal SIG${received_signal}");
+    $OUTPUT->debug_msg(2, "Retriggering signal SIG${received_signal}");
     return kill($received_signal, $$);
 }
 
