@@ -1,7 +1,8 @@
 # Hey emacs! This is a -*- Perl -*- script!
 # Lintian::Util -- Perl utility functions for lintian
 
-# Copyright (C) 1998 Christian Schwarz
+# Copyright © 1998 Christian Schwarz
+# Copyright © 2020 Felix Lechner
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -65,7 +66,7 @@ BEGIN {
           lstrip
           rstrip
           copy_dir
-          sort_file_index
+          human_bytes
           gunzip_file
           open_gz
           gzip
@@ -81,6 +82,7 @@ BEGIN {
           drop_relative_prefix
           dequote_name
           pipe_tee
+          read_md5sums
           untaint
           $PKGNAME_REGEX
           $PKGREPACK_REGEX
@@ -99,6 +101,7 @@ use Lintian::Relation::Version qw(versions_equal versions_comparator);
 use constant EMPTY => q{};
 use constant SPACE => q{ };
 use constant COLON => q{:};
+use constant BACKSLASH => q{\\};
 use constant NEWLINE => qq{\n};
 
 # read up to 40kB at a time.  this happens to be 4096 "tar records"
@@ -108,6 +111,17 @@ use constant NEWLINE => qq{\n};
 # record.
 use constant READ_SIZE => 4096 * 20 * 512;
 
+# preload cache for common permission strings
+# call overhead o perm2oct was measurable on chromium-browser/32.0.1700.123-2
+# load time went from ~1.5s to ~0.1s; of 115363 paths, only 306 were uncached
+my %OCTAL_LOOKUP = map { $_ => perm2oct($_) } (
+    '-rw-r--r--', # standard (non-executable) file
+    '-rwxr-xr-x', # standard executable file
+    'drwxr-xr-x', # standard dir perm
+    'drwxr-sr-x', # standard dir perm with suid (lintian-lab on lintian.d.o)
+    'lrwxrwxrwx', # symlinks
+);
+
 =head1 NAME
 
 Lintian::Util - Lintian utility functions
@@ -514,40 +528,46 @@ Examples:
 =cut
 
 sub perm2oct {
-    my ($t) = @_;
+    my ($text) = @_;
 
-    my $o = 0;
+    my $lookup = $OCTAL_LOOKUP{$text};
+    return $lookup
+      if defined $lookup;
+
+    my $octal = 0;
 
     # Types:
     #  file (-), block/character device (b & c), directory (d),
     #  hardlink (h), symlink (l), named pipe (p).
     if (
-        $t !~ m/^   [-bcdhlp]                # file type
+        $text !~ m/^   [-bcdhlp]                # file type
                     ([-r])([-w])([-xsS])     # user
                     ([-r])([-w])([-xsS])     # group
                     ([-r])([-w])([-xtT])     # other
-               /xsmo
+               /xsm
     ) {
-        croak "$t does not appear to be a permission string";
+        croak "$text does not appear to be a permission string";
     }
 
-    $o += 00400 if $1 eq 'r';   # owner read
-    $o += 00200 if $2 eq 'w';   # owner write
-    $o += 00100 if $3 eq 'x';   # owner execute
-    $o += 04000 if $3 eq 'S';   # setuid
-    $o += 04100 if $3 eq 's';   # setuid + owner execute
-    $o += 00040 if $4 eq 'r';   # group read
-    $o += 00020 if $5 eq 'w';   # group write
-    $o += 00010 if $6 eq 'x';   # group execute
-    $o += 02000 if $6 eq 'S';   # setgid
-    $o += 02010 if $6 eq 's';   # setgid + group execute
-    $o += 00004 if $7 eq 'r';   # other read
-    $o += 00002 if $8 eq 'w';   # other write
-    $o += 00001 if $9 eq 'x';   # other execute
-    $o += 01000 if $9 eq 'T';   # stickybit
-    $o += 01001 if $9 eq 't';   # stickybit + other execute
+    $octal += 00400 if $1 eq 'r';   # owner read
+    $octal += 00200 if $2 eq 'w';   # owner write
+    $octal += 00100 if $3 eq 'x';   # owner execute
+    $octal += 04000 if $3 eq 'S';   # setuid
+    $octal += 04100 if $3 eq 's';   # setuid + owner execute
+    $octal += 00040 if $4 eq 'r';   # group read
+    $octal += 00020 if $5 eq 'w';   # group write
+    $octal += 00010 if $6 eq 'x';   # group execute
+    $octal += 02000 if $6 eq 'S';   # setgid
+    $octal += 02010 if $6 eq 's';   # setgid + group execute
+    $octal += 00004 if $7 eq 'r';   # other read
+    $octal += 00002 if $8 eq 'w';   # other write
+    $octal += 00001 if $9 eq 'x';   # other execute
+    $octal += 01000 if $9 eq 'T';   # stickybit
+    $octal += 01001 if $9 eq 't';   # stickybit + other execute
 
-    return $o;
+    $OCTAL_LOOKUP{$text} = $octal;
+
+    return $octal;
 }
 
 =item run_cmd([OPTS, ]COMMAND[, ARGS...])
@@ -700,26 +720,26 @@ sub copy_dir {
     return run_cmd('cp', '-a', '--reflink=auto', '--', @_);
 }
 
-=item sort_file_index (STRING)
-
-Sorts the file index data given in STRING and returns the sorted
-result, also in a string.
+=item human_bytes(SIZE)
 
 =cut
 
-sub sort_file_index {
-    my ($input) = @_;
+sub human_bytes {
+    my ($size) = @_;
 
-    my @unsorted = split(NEWLINE, $input);
+    my @units = ('B', 'kiB', 'MiB', 'GiB');
 
-    # sorts according to LC_ALL=C
-    my @sorted
-      = sort { (split(SPACE, $a))[5] cmp(split(SPACE, $b))[5] } @unsorted;
+    my $unit = shift @units;
 
-    my $output = EMPTY;
-    $output .= $_ . NEWLINE for @sorted;
+    while ($size > 1536 && @units) {
 
-    return $output;
+        $size /= 1024;
+        $unit = shift @units;
+    }
+
+    my $human = sprintf('%.0f %s', $size, $unit);
+
+    return $human;
 }
 
 =item gunzip_file (IN, OUT)
@@ -1006,9 +1026,10 @@ Remove an initial ./ from STRING, if present
 sub drop_relative_prefix {
     my ($name) = @_;
 
-    $name =~ s,^\.?/,,;
+    my $copy = $name;
+    $copy =~ s{^\./}{}s;
 
-    return $name;
+    return $copy;
 }
 
 =item dequote_name(STR, REMOVESLASH)
@@ -1022,6 +1043,8 @@ Remove initial ./ by default
 
 sub dequote_name {
     my ($name, $slsd) = @_;
+    return
+      unless defined $name;
     $slsd = 1 unless defined $slsd;
     $name =~ s,^\.?/,, if $slsd;
     # Optimise for the case where the filename does not contain
@@ -1287,6 +1310,93 @@ sub pipe_tee {
     return 1;
 }
 
+=item read_md5sums
+
+Untaint VALUE
+
+=cut
+
+sub read_md5sums {
+    my ($text) = @_;
+
+    # make a local copy
+    my $buffer = $text;
+
+    my %checksums;
+    my @errors;
+
+  TRY_AGAIN:
+
+    # start with checksum; processing style inspired by IO::Async::Stream
+    while ($buffer =~ s/^((?:\\)?\S{32}) [ *]//) {
+
+        my $checksum = $1;
+        my $path = EMPTY;
+        my $problematic = 0;
+
+        # leading slash in checksum indicates an escaped name
+        $problematic = 1
+          if $checksum =~ s{^\\}{};
+
+        # read up until the next newline
+        while ($buffer =~ s/^([^\n]+)\n//) {
+            my $section = $1;
+
+            # done if newline means end of record
+            unless ($problematic) {
+                $path .= $section;
+                last;
+            }
+
+            # split into individual characters
+            my @array = split(//, $section);
+
+# https://www.gnu.org/software/coreutils/manual/html_node/md5sum-invocation.html
+            my $escaped = 0;
+            for my $char (@array) {
+
+                # take next character verbatim
+                if ($escaped || $char ne BACKSLASH) {
+                    $path .= $char;
+                    $escaped = 0;
+                    next;
+                }
+
+                # start escape sequence
+                if ($char eq BACKSLASH) {
+                    $escaped = 1;
+                    next;
+                }
+            }
+
+            # do not stop inside an escape sequence
+            last
+              unless $escaped;
+
+            # add the newline that was stripped and read more
+            $path .= NEWLINE;
+        }
+
+        push(@errors, "Empty name for checksum $checksum")
+          unless length $path;
+
+        $checksums{$path} = $checksum;
+    }
+
+    if ($buffer =~ s/([^\n]*)(?:\n|\z)//) {
+        my $unreadable = $1;
+
+        # this skips empty lines
+        push(@errors, "Odd text: $unreadable")
+          if length $unreadable;
+
+        goto TRY_AGAIN
+          if length $buffer;
+    }
+
+    return (\%checksums, \@errors);
+}
+
 =item untaint(VALUE)
 
 Untaint VALUE
